SystemOrganization addCategory: #FuelPackageLoader!
SystemOrganization addCategory: #'FuelPackageLoader-Mappers'!
SystemOrganization addCategory: #'FuelPackageLoader-Metacello'!

!Association methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/24/2011 11:26'!
rebindOn: aCompiledMethod at: index
	"If the CompiledMethod was saved with a reference to a class that does exist, then its calculated binding will be nil. In that case we prefer to keep the original association."
	(aCompiledMethod methodClass bindingOf: key)
		ifNotNil: [:aBinding |
			aCompiledMethod literalAt: index put: aBinding]! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 21:16'!
cleanUpClassVariables

	self classPool keys do: [ :key | classPool at: key put: nil ]! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 21:41'!
cleanUpInstanceVariables

	Class instSize + 1 to: self class instSize do:
		[:offset | self instVarAt: offset put: nil].
! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MarianoMartinezPeck 9/23/2011 16:44'!
copyWithLocalProtocol

	| copy |
	copy := self copy.

	copy organization: organization deepCopy.
	copy environment: environment.
	copy basicCategory: category.	
		
	copy traitComposition: self traitComposition.
	copy classSide traitComposition: self classSide traitComposition.
	
	copy removeForeignProtocol.
	copy classSide removeForeignProtocol.
	
	copy cleanUpClassVariables.
	copy cleanUpInstanceVariables.
	
	^copy! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/29/2011 01:30'!
initializeOnLoad

	(self class includesSelector: #initialize)
		ifTrue: [self initialize]! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/29/2011 01:29'!
rebindOnLoad

	self rebindMethodDictionary.
	self class rebindMethodDictionary! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/29/2011 01:29'!
registerOnLoad

	superclass addSubclass: self. "Note that aClass could already be between its superclass subclasses, #addSubclass: is prepared for that case."
	Smalltalk globals at: self name put: self.
	Smalltalk globals organization classify: self name under: self basicCategory.! !

FLFullGeneralMapper subclass: #FLPackageLoaderGeneralMapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader-Mappers'!

!FLPackageLoaderGeneralMapper commentStamp: 'MartinDias 8/24/2011 00:08' prior: 0!
I am like the default mapper but with a special behavior with classes. I store a copy each class with just the local protocol.!

!FLPackageLoaderGeneralMapper methodsFor: 'visiting' stamp: 'MartinDias 1/9/2012 15:29'!
visitClass: aClass  

	(Smalltalk globals includes: aClass)
		ifTrue: [ self visitSubstitution: aClass by: aClass copyWithLocalProtocol ]
		ifFalse: [ super visitClass: aClass ]! !

!CompiledMethod methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 02:10'!
rebindLiteralAt: index

	(self literalAt: index) rebindOn: self at: index! !

!CompiledMethod methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 02:07'!
rebindLiterals 

	1 to: self numLiterals-2
		do: [ :index | self rebindLiteralAt: index ]! !

FLAnalyzer subclass: #FLPackageAnalyzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageAnalyzer methodsFor: 'defaults' stamp: 'MarianoMartinezPeck 12/14/2011 20:44'!
generalMapper

	^FLPackageLoaderGeneralMapper new! !

!TClassAndTraitDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 00:27'!
removeCategorySilently: aCategory

	SystemChangeNotifier uniqueInstance doSilently: [ self removeCategory: aCategory ]! !

!ClassOrganizer methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:32'!
removeCategorySilently: cat 

	super removeCategory: cat! !

!ClassOrganizer methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:49'!
removeElementSilently: element

	^super removeElement: element! !

!Trait methodsFor: '*FuelPackageLoader' stamp: 'PavelKrivanek 4/11/2012 17:21'!
rebindMethodDictionary

	self methodDict valuesDo: [ :aCompiledMethod | 
		aCompiledMethod 
			methodClass: self;
			rebindLiterals. ]! !

!Trait methodsFor: '*FuelPackageLoader' stamp: 'PavelKrivanek 4/11/2012 17:19'!
registerOnLoad

	Smalltalk globals at: self name put: self.
	Smalltalk globals organization classify: self name under: self basicCategory.! !

!Behavior methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 15:35'!
rebindMethodDictionary

	self methodDict valuesDo: [ :aCompiledMethod | 
		aCompiledMethod 
			methodClass: self;
			rebindLiterals. ]! !

Object subclass: #FLMetacelloStore
	instanceVariableNames: 'prefix capturing capturedPackageNames allPackagesTogetherStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader-Metacello'!

FLMetacelloStore class
	instanceVariableNames: 'default'!

!FLMetacelloStore commentStamp: 'MartinDias 9/29/2011 04:16' prior: 0!
I capture and export in Fuel files a Metacello load.

Example: loading Seaside 3

1) Install Fuel in a CLEAN Pharo 1.3 image:
Gofer it squeaksource: 'Fuel'; 
	package: 'Fuel';
	package: 'FuelPackageLoader';
	load.

2) Load the configuration you want to capture in this way:
FLMetacelloStore default 
	capturePackagesDuring: [
		Gofer new squeaksource: 'MetacelloRepository'; package: 'ConfigurationOfSeaside30'; load.
		(Smalltalk at: #ConfigurationOfSeaside30) project latestVersion load: #(Core)] 
	prefixed: 'seaside'.

= Load =

1) Install Fuel:
Gofer it squeaksource: 'Fuel'; 
	package: 'Fuel';
	package: 'FuelPackageLoader';
	load.

2) Now run the loading script generated: fileIn './FuelMetacello/seaside-load-.st'."

3) "Start the server"
WAComancheAdaptor startOn: 8088.!

FLMetacelloStore class
	instanceVariableNames: 'default'!

!FLMetacelloStore classSide methodsFor: 'instance creation' stamp: 'MartinDias 9/29/2011 02:26'!
default

	^default ifNil: [default := self new]! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 02:45'!
allPackagesTogetherFileName

	^prefix, '-all-packages-together.fuel'! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 02:54'!
capturePackages: packageNames

	packageNames
		do: [:packageName |
			capturedPackageNames add: packageName.
			FLPackageStore new storeOn: allPackagesTogetherStream packageNamed: packageName ]
		displayingProgress: [:packageName | 'Capturing ', packageName ]
	! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 03:41'!
capturePackagesDuring: aBlock prefixed: aPrefix

	capturedPackageNames := OrderedCollection new.
	prefix := aPrefix.

	self outputFileNamed:  self allPackagesTogetherFileName do: [:aStream |
		capturing := true.
		allPackagesTogetherStream := aStream binary.
		aBlock value.
		capturing := false ].

	self writeLoadingScript.
! !

!FLMetacelloStore methodsFor: 'initialize-release' stamp: 'MartinDias 9/29/2011 02:26'!
initialize 

	super initialize.
	capturing := false.! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 03:42'!
loadingScriptFileName
	
	^prefix, '-load.st'! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 03:41'!
loadingScriptString

	^ '
	| packageFileNames |
	packageFileNames := ', capturedPackageNames asArray printString, '.
	
	(FileDirectory default directoryNamed: ''FuelMetacello'') 
			oldFileNamed: ', self allPackagesTogetherFileName printString, ' 
			do: [ :aStream | 
				aStream binary.
				packageFileNames 
					do: [ :packageFileName | FLPackageLoader new loadFrom: aStream ]
					displayingProgress: [ :packageFileName | ''Fuel importing '', packageFileName ]]'! !

!FLMetacelloStore methodsFor: 'streaming' stamp: 'MartinDias 9/29/2011 02:26'!
outputDirectory

	^(FileDirectory default directoryNamed: 'FuelMetacello') 
		assureExistence;
		yourself
! !

!FLMetacelloStore methodsFor: 'streaming' stamp: 'MartinDias 9/29/2011 02:26'!
outputFileNamed: aFileName do: aBlock

	^self outputDirectory forceNewFileNamed: aFileName do: aBlock! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 02:26'!
packageFileNameFor: packageName
 
	^prefix, '-', packageName, '.fuel'! !

!FLMetacelloStore methodsFor: 'listening' stamp: 'MartinDias 9/29/2011 02:26'!
packagesHasBeenLoaded: packageNames

	capturing ifTrue: [ self capturePackages: packageNames]! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 9/29/2011 03:43'!
writeLoadingScript

	self 
		outputFileNamed: self loadingScriptFileName 
		do: [ :scriptStream | scriptStream nextPutAll: self loadingScriptString ].! !

Object subclass: #FLMethodExtension
	instanceVariableNames: 'method category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLMethodExtension classSide methodsFor: 'instance creation' stamp: 'MartinDias 9/29/2011 01:53'!
with: aCompiledMethod

	^ self basicNew
		initializeWith: aCompiledMethod;
		yourself ! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
category
	^ category! !

!FLMethodExtension methodsFor: 'initialize-release' stamp: 'MartinDias 9/29/2011 01:52'!
initializeWith: aCompiledMethod

	self initialize.
	method := aCompiledMethod.
	category := aCompiledMethod category.! !

!FLMethodExtension methodsFor: 'loading' stamp: 'MartinDias 9/22/2011 15:39'!
load

	method rebindLiterals.
	method methodClass addSelectorSilently: method selector withMethod: method.
	method methodClass organization classify: method selector under: category.
	! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
method
	^ method! !

Object subclass: #FLPackage
	instanceVariableNames: 'extensionMethods classes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

FLPackage class
	instanceVariableNames: 'initializersEnabled'!

FLPackage class
	instanceVariableNames: 'initializersEnabled'!

!FLPackage classSide methodsFor: 'private' stamp: 'MartinDias 8/22/2011 02:53'!
classesLocalTo: packageInfo
	^ packageInfo classes! !

!FLPackage classSide methodsFor: 'private' stamp: 'MartinDias 8/21/2011 19:21'!
extensionMethodsTo: packageInfo
	^ packageInfo extensionMethods 
		collect: [ :reference | FLMethodExtension with: reference compiledMethod ]! !

!FLPackage classSide methodsFor: 'initializers' stamp: 'PavelKrivanek 4/11/2012 17:15'!
initializersEnabled

	^ initializersEnabled ifNil: [ initializersEnabled := true ].! !

!FLPackage classSide methodsFor: 'initializers' stamp: 'PavelKrivanek 4/11/2012 17:15'!
initializersEnabled: aBoolean

	initializersEnabled := aBoolean! !

!FLPackage classSide methodsFor: 'instance creation' stamp: 'MartinDias 9/29/2011 00:09'!
named: packageName

	| packageInfo |
	packageInfo := PackageInfo named: packageName.
	^ self basicNew
		initializeWithClasses: (self classesLocalTo: packageInfo) 
		extensionMethods: (self extensionMethodsTo: packageInfo);
		yourself! !

!FLPackage classSide methodsFor: 'instance creation' stamp: 'PavelKrivanek 4/11/2012 17:15'!
withPackages: packageNames

	| classes methods packageInfo |
	classes := OrderedCollection new.
	methods := OrderedCollection new.
	packageNames do: [:packageName |
		packageInfo := PackageInfo named: packageName.
		classes addAll: (self classesLocalTo: packageInfo).
		methods addAll: (self extensionMethodsTo: packageInfo)].
	
	^ self basicNew
		initializeWithClasses: classes
		extensionMethods: methods;
		yourself! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 19:10'!
classes
	^ classes! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:35'!
extensionMethods
	^ extensionMethods! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 9/29/2011 00:10'!
initializeWithClasses: someClasses extensionMethods: someExtensionMethods

	self initialize.
	classes := someClasses.
	extensionMethods := someExtensionMethods.! !

!FLPackage methodsFor: 'loading' stamp: 'PavelKrivanek 4/11/2012 17:16'!
load
	self classes do: [:aClass | aClass registerOnLoad].
	Smalltalk globals flushClassNameCache.
	self classes do: [:aClass | aClass rebindOnLoad].
	self classes do: [:aClass | 
		self class initializersEnabled ifTrue: [aClass initializeOnLoad].
		self notifyClassAddition: aClass.].
	self loadExtensionMethods.
	Behavior flushObsoleteSubclasses! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 3/26/2011 04:44'!
loadExtensionMethods
	
	extensionMethods do: [ :extensionMethod | extensionMethod load ]! !

!FLPackage methodsFor: 'loading' stamp: 'PavelKrivanek 4/11/2012 17:25'!
notifyClassAddition: aClass

	 | category |

	category := aClass category ifNil: 'UNKNOWN'.
	SystemChangeNotifier uniqueInstance classAdded: aClass inCategory: category asSymbol.

! !

Object subclass: #FLPackageLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageLoader methodsFor: 'loading' stamp: 'MartinDias 9/13/2011 22:01'!
loadFrom: aStream 

	| package |
	package := (FLMaterializer newDefault materializeFrom: aStream) root.
	package load.
	^ package.! !

Object subclass: #FLPackageStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageStore methodsFor: 'storing' stamp: 'MartinDias 11/30/2011 01:15'!
storeOn: aStream packageNamed: name 

	| package |
	package := FLPackage named: name.
	(FLSerializer newDefault)
		analyzer: (FLPackageAnalyzer newDefault 
			considerInternalAll: package classes;
			yourself);
		serialize: package on: aStream.! !

!Object methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 02:12'!
rebindOn: aCompiledMethod at: index
	"Nothing to do."! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 02:42'!
foreignCategories
	
	^self organization categories select: [ :aCategory | aCategory beginsWith: '*' ]! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:50'!
removeCategorySilently: aCategory

	(organization listAtCategoryNamed: aCategory) do: [ :aSelector | self removeSelectorSilently: aSelector ].
	organization removeCategorySilently: aCategory.
! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 03:05'!
removeForeignProtocol

	self foreignCategories do: [ :aCategory |
		self removeCategorySilently: aCategory ]! !
